@@ -5,6 +5,7 @@ import math
 import jax
 import jax.experimental.sparse as jax_sparse
 import jax.numpy as jnp
+from absl import logging
 from jax import lax
 from jax import nn as jnn
 from jax.experimental.pallas.ops.tpu.splash_attention import (
@@ -395,6 +396,8 @@ def depthwise_conv(
     feature_group_count = (
         inputs.shape[-1] if data_format == "channels_last" else inputs.shape[1]
     )
+    kernel = convert_to_tensor(kernel)
+    inputs = convert_to_tensor(inputs)
     kernel = jnp.reshape(
         kernel,
         kernel.shape[:-2] + (1, feature_group_count * kernel.shape[-1]),
@@ -707,7 +710,9 @@ def ctc_loss(target, output, target_length, output_length, mask_index=0):
     logprobs_phi = logprobs[:, :, mask_index : mask_index + 1]  # [B, T, 1]
     logprobs_phi = jnp.transpose(logprobs_phi, (1, 0, 2))  # [T, B, 1]
 
-    _one_hot = jax.nn.one_hot(target, num_classes=num_classes)  # [B, N, K]
+    _one_hot = jax.nn.one_hot(
+        target, num_classes=num_classes, dtype=logprobs.dtype
+    )  # [B, N, K]
     logprobs_emit = jnp.einsum("btk,bnk->btn", logprobs, _one_hot)
     logprobs_emit = jnp.transpose(logprobs_emit, (1, 0, 2))  # [T, B, N]
 
@@ -764,7 +769,11 @@ def ctc_loss(target, output, target_length, output_length, mask_index=0):
 
     # extract per_seq_loss
     # [B, N+1]
-    _one_hot = jax.nn.one_hot(label_lengths, num_classes=max_label_length + 1)
+    _one_hot = jax.nn.one_hot(
+        label_lengths,
+        num_classes=max_label_length + 1,
+        dtype=logalpha_phi_last.dtype,
+    )
     per_seq_loss = -jnp.einsum("bn,bn->b", logalpha_phi_last, _one_hot)
     return per_seq_loss
 
@@ -1240,6 +1249,12 @@ def dot_product_attention(
             f"Received: query.shape={query.shape}, key.shape={key.shape}, "
             f"value.shape={value.shape}."
         )
+    compute_dtype = backend.result_type(query.dtype, key.dtype, value.dtype)
+    query = cast(query, compute_dtype)
+    key = cast(key, compute_dtype)
+    value = cast(value, compute_dtype)
+    if bias is not None:
+        bias = convert_to_tensor(bias, dtype=compute_dtype)
 
     # Check platform
     platform = jax.devices()[0].platform
@@ -1256,6 +1271,9 @@ def dot_product_attention(
     # TPU-specific flash attention path
     if is_tpu and flash_attention:
         # Get sharding parameters from distribution context
+        head_shards = 1
+        # Typically keep q_seq_shards=1 for best performance
+        q_seq_shards = 1
         try:
             from keras.src.distribution.distribution_lib import ModelParallel
             from keras.src.distribution.distribution_lib import (
@@ -1270,12 +1288,12 @@ def dot_product_attention(
                     model_dim_index = mesh.axis_names.index("model")
                     # Set head_shards based on the model dimension of the mesh
                     head_shards = mesh.shape[model_dim_index]
-                    # Typically keep q_seq_shards=1 for best performance
-                    q_seq_shards = 1
         except (ImportError, ValueError, AttributeError):
             # Use default values if detection fails
-            head_shards = 1
-            q_seq_shards = 1
+            logging.exception(
+                "Failed to determine distribution context for sharding. "
+                "Using default head_shards=1 and q_seq_shards=1."
+            )
         # Transpose to ('batch', 'heads', 'length', 'head_dim')
         query_tpu_layout = jnp.transpose(query, axes=(0, 2, 1, 3))
         key_tpu_layout = jnp.transpose(key, axes=(0, 2, 1, 3))
@@ -1328,24 +1346,17 @@ def dot_product_attention(
             # Transpose output back to Keras layout
             return jnp.transpose(output, axes=(0, 2, 1, 3))
         except Exception:
+            logging.exception(
+                "Failed to apply Splash kernel for flash attention. "
+                "Falling back to JAX native dot_product_attention."
+            )
             flash_attention = False
 
     # JAX native dot_product_attention for GPU or fallback for TPU
     if hasattr(jax.nn, "dot_product_attention"):
-        try:
-            return jax.nn.dot_product_attention(
-                query,
-                key,
-                value,
-                bias=bias,
-                mask=mask,
-                scale=scale,
-                is_causal=is_causal,
-                implementation="cudnn" if flash_attention else "xla",
-            )
-        except Exception:
-            # If flash attention fails, fall back to XLA implementation
-            if flash_attention:
+        impls = ["cudnn", "xla"] if flash_attention else ["xla"]
+        for impl in impls:
+            try:
                 return jax.nn.dot_product_attention(
                     query,
                     key,
@@ -1354,9 +1365,13 @@ def dot_product_attention(
                     mask=mask,
                     scale=scale,
                     is_causal=is_causal,
-                    implementation="xla",
+                    implementation=impl,
+                )
+            except Exception:
+                logging.exception(
+                    f"Failed to apply {impl} implementation of "
+                    "jax.nn.dot_product_attention."
                 )
-            raise
 
     if flash_attention:
         raise RuntimeError(
@@ -1398,3 +1413,46 @@ def dot_product_attention(
     )
     encoded = vmapped_fn(query, key, value, bias, mask, is_causal, scale)
     return jnp.reshape(encoded, output_shape)
+
+
+def unfold(input, kernel_size, dilation=1, padding=0, stride=1):
+    """JAX implementation of Unfold.
+    Extract sliding local blocks from a **NCHW** batched image tensor.
+
+    Args:
+        input: 4-D tensor, shape (N, C, H, W)  **required**.
+        kernel_size: int or (kH, kW)
+        dilation: int or (dH, dW), default 1
+        padding: int or (pH, pW), default 0
+        stride: int or (sH, sW), default 1
+
+    Returns:
+        3-D tensor, shape (N, C*kH*kW, L)
+    """
+
+    def _pair(x):
+        return (x, x) if isinstance(x, int) else x
+
+    k = _pair(kernel_size)
+    d = _pair(dilation)
+    p = _pair(padding)
+    s = _pair(stride)
+
+    N, C, H, W = input.shape
+
+    # ---- padding ----
+    if any(_ > 0 for _ in p):
+        input = jnp.pad(input, ((0, 0), (0, 0), (p[0], p[0]), (p[1], p[1])))
+
+    patches = lax.conv_general_dilated_patches(
+        input,
+        filter_shape=k,
+        window_strides=s,
+        padding="VALID",  # has padde
+        rhs_dilation=d,
+        dimension_numbers=("NCHW", "OIHW", "NCHW"),  # only support 'NCHW'
+    )  # shape: (N, C*kH*kW, oH, oW)
+
+    # ---- reshape -> (N, C*kH*kW, L) ----
+    _, CKK, oH, oW = patches.shape
+    return patches.reshape(N, CKK, oH * oW)
