@@ -13,6 +13,34 @@ RESIZE_INTERPOLATIONS = (
     "lanczos5",
     "bicubic",
 )
+AFFINE_TRANSFORM_INTERPOLATIONS = {  # map to order
+    "nearest": 0,
+    "bilinear": 1,
+}
+AFFINE_TRANSFORM_FILL_MODES = {
+    "constant",
+    "nearest",
+    "wrap",
+    "mirror",
+    "reflect",
+}
+MAP_COORDINATES_FILL_MODES = {
+    "constant",
+    "nearest",
+    "wrap",
+    "mirror",
+    "reflect",
+}
+SCALE_AND_TRANSLATE_METHODS = {
+    "linear",
+    "bilinear",
+    "trilinear",
+    "cubic",
+    "bicubic",
+    "tricubic",
+    "lanczos3",
+    "lanczos5",
+}
 
 
 def rgb_to_grayscale(images, data_format=None):
@@ -367,7 +395,7 @@ def resize(
     return _resize(images, size, method=interpolation, antialias=antialias)
 
 
-def compute_weight_mat(
+def _compute_weight_mat(
     input_size, output_size, scale, translation, kernel, antialias
 ):
     dtype = np.result_type(scale, translation)
@@ -410,32 +438,11 @@ def compute_weight_mat(
 
 
 def _resize(image, shape, method, antialias):
-    def _fill_triangle_kernel(x):
-        return np.maximum(0, 1 - np.abs(x))
-
-    def _fill_keys_cubic_kernel(x):
-        out = ((1.5 * x - 2.5) * x) * x + 1.0
-        out = np.where(x >= 1.0, ((-0.5 * x + 2.5) * x - 4.0) * x + 2.0, out)
-        return np.where(x >= 2.0, 0.0, out)
-
-    def _fill_lanczos_kernel(radius, x):
-        y = radius * np.sin(np.pi * x) * np.sin(np.pi * x / radius)
-        out = np.where(
-            x > 1e-3, np.divide(y, np.where(x != 0, np.pi**2 * x**2, 1)), 1
-        )
-        return np.where(x > radius, 0.0, out)
-
     if method == "nearest":
         return _resize_nearest(image, shape)
-    elif method == "bilinear":
-        kernel = _fill_triangle_kernel
-    elif method == "lanczos3":
-        kernel = lambda x: _fill_lanczos_kernel(3.0, x)
-    elif method == "lanczos5":
-        kernel = lambda x: _fill_lanczos_kernel(5.0, x)
-    elif method == "bicubic":
-        kernel = _fill_keys_cubic_kernel
     else:
+        kernel = _kernels.get(method, None)
+    if kernel is None:
         raise ValueError("Unknown resize method")
 
     spatial_dims = tuple(
@@ -473,6 +480,34 @@ def _resize_nearest(x, output_shape):
     return x
 
 
+def _fill_triangle_kernel(x):
+    return np.maximum(0, 1 - np.abs(x))
+
+
+def _fill_keys_cubic_kernel(x):
+    out = ((1.5 * x - 2.5) * x) * x + 1.0
+    out = np.where(x >= 1.0, ((-0.5 * x + 2.5) * x - 4.0) * x + 2.0, out)
+    return np.where(x >= 2.0, 0.0, out)
+
+
+def _fill_lanczos_kernel(radius, x):
+    y = radius * np.sin(np.pi * x) * np.sin(np.pi * x / radius)
+    out = np.where(
+        x > 1e-3, np.divide(y, np.where(x != 0, np.pi**2 * x**2, 1)), 1
+    )
+    return np.where(x > radius, 0.0, out)
+
+
+_kernels = {
+    "linear": _fill_triangle_kernel,
+    "bilinear": _fill_triangle_kernel,  # For `resize`.
+    "cubic": _fill_keys_cubic_kernel,
+    "bicubic": _fill_keys_cubic_kernel,  # For `resize`.
+    "lanczos3": lambda x: _fill_lanczos_kernel(3.0, x),
+    "lanczos5": lambda x: _fill_lanczos_kernel(5.0, x),
+}
+
+
 def _scale_and_translate(
     x, output_shape, spatial_dims, scale, translation, kernel, antialias
 ):
@@ -492,9 +527,9 @@ def _scale_and_translate(
         d = d % x.ndim
         m, n = input_shape[d], output_shape[d]
 
-        w = compute_weight_mat(
+        w = _compute_weight_mat(
             m, n, scale[i], translation[i], kernel, antialias
-        ).astype(np.float32)
+        ).astype(output.dtype)
         output = np.tensordot(output, w, axes=(d, 0))
         output = np.moveaxis(output, -1, d)
 
@@ -504,19 +539,6 @@ def _scale_and_translate(
     return output
 
 
-AFFINE_TRANSFORM_INTERPOLATIONS = {  # map to order
-    "nearest": 0,
-    "bilinear": 1,
-}
-AFFINE_TRANSFORM_FILL_MODES = {
-    "constant",
-    "nearest",
-    "wrap",
-    "mirror",
-    "reflect",
-}
-
-
 def affine_transform(
     images,
     transform,
@@ -538,6 +560,7 @@ def affine_transform(
             f"{AFFINE_TRANSFORM_FILL_MODES}. Received: fill_mode={fill_mode}"
         )
 
+    images = convert_to_tensor(images)
     transform = convert_to_tensor(transform)
 
     if len(images.shape) not in (3, 4):
@@ -553,10 +576,11 @@ def affine_transform(
             f"transform.shape={transform.shape}"
         )
 
-    # scipy.ndimage.map_coordinates lacks support for half precision.
-    input_dtype = images.dtype
-    if input_dtype == "float16":
-        images = images.astype("float32")
+    # `scipy.ndimage.map_coordinates` lacks support for float16 and bfloat16.
+    input_dtype = backend.standardize_dtype(images.dtype)
+    compute_dtype = backend.result_type(input_dtype, "float32")
+    images = images.astype(compute_dtype)
+    transform = transform.astype(compute_dtype)
 
     # unbatched case
     need_squeeze = False
@@ -600,7 +624,7 @@ def affine_transform(
     # transform the indices
     coordinates = np.einsum("Bhwij, Bjk -> Bhwik", indices, transform)
     coordinates = np.moveaxis(coordinates, source=-1, destination=1)
-    coordinates += np.reshape(offset, newshape=(*offset.shape, 1, 1, 1))
+    coordinates += np.reshape(offset, (*offset.shape, 1, 1, 1))
 
     # apply affine transformation
     affined = np.stack(
@@ -621,9 +645,7 @@ def affine_transform(
         affined = np.transpose(affined, (0, 3, 1, 2))
     if need_squeeze:
         affined = np.squeeze(affined, axis=0)
-    if input_dtype == "float16":
-        affined = affined.astype(input_dtype)
-    return affined
+    return affined.astype(input_dtype)
 
 
 def perspective_transform(
@@ -736,6 +758,14 @@ def perspective_transform(
 
 
 def compute_homography_matrix(start_points, end_points):
+    start_points = convert_to_tensor(start_points)
+    end_points = convert_to_tensor(end_points)
+    dtype = backend.result_type(start_points.dtype, end_points.dtype, float)
+    # `np.linalg.solve` lacks support for float16 and bfloat16.
+    compute_dtype = backend.result_type(dtype, "float32")
+    start_points = start_points.astype(dtype)
+    end_points = end_points.astype(dtype)
+
     start_x1, start_y1 = start_points[:, 0, 0], start_points[:, 0, 1]
     start_x2, start_y2 = start_points[:, 1, 0], start_points[:, 1, 1]
     start_x3, start_y3 = start_points[:, 2, 0], start_points[:, 2, 1]
@@ -870,20 +900,11 @@ def compute_homography_matrix(start_points, end_points):
         axis=-1,
     )
     target_vector = np.expand_dims(target_vector, axis=-1)
-
+    coefficient_matrix = coefficient_matrix.astype(compute_dtype)
+    target_vector = target_vector.astype(compute_dtype)
     homography_matrix = np.linalg.solve(coefficient_matrix, target_vector)
     homography_matrix = np.reshape(homography_matrix, [-1, 8])
-
-    return homography_matrix
-
-
-MAP_COORDINATES_FILL_MODES = {
-    "constant",
-    "nearest",
-    "wrap",
-    "mirror",
-    "reflect",
-}
+    return homography_matrix.astype(dtype)
 
 
 def map_coordinates(
@@ -937,10 +958,14 @@ def map_coordinates(
         )
     else:
         padded = np.pad(inputs, padding, mode=pad_mode)
+
+    # `scipy.ndimage.map_coordinates` lacks support for float16 and bfloat16.
+    if backend.is_float_dtype(padded.dtype):
+        padded = padded.astype("float32")
     result = scipy.ndimage.map_coordinates(
         padded, shifted_coords, order=order, mode=fill_mode, cval=fill_value
     )
-    return result
+    return result.astype(inputs.dtype)
 
 
 def gaussian_blur(
@@ -966,7 +991,11 @@ def gaussian_blur(
     images = convert_to_tensor(images)
     kernel_size = convert_to_tensor(kernel_size)
     sigma = convert_to_tensor(sigma)
-    input_dtype = images.dtype
+    input_dtype = backend.standardize_dtype(images.dtype)
+    # `scipy.signal.convolve2d` lacks support for float16 and bfloat16.
+    compute_dtype = backend.result_type(input_dtype, "float32")
+    images = images.astype(compute_dtype)
+    sigma = sigma.astype(compute_dtype)
 
     if len(images.shape) not in (3, 4):
         raise ValueError(
@@ -1009,8 +1038,7 @@ def gaussian_blur(
         blurred_images = np.transpose(blurred_images, (0, 3, 1, 2))
     if need_squeeze:
         blurred_images = np.squeeze(blurred_images, axis=0)
-
-    return blurred_images
+    return blurred_images.astype(input_dtype)
 
 
 def elastic_transform(
@@ -1135,3 +1163,40 @@ def elastic_transform(
     transformed_images = transformed_images.astype(input_dtype)
 
     return transformed_images
+
+
+def scale_and_translate(
+    images,
+    output_shape,
+    scale,
+    translation,
+    spatial_dims,
+    method,
+    antialias=True,
+):
+    if method not in SCALE_AND_TRANSLATE_METHODS:
+        raise ValueError(
+            "Invalid value for argument `method`. Expected of one "
+            f"{SCALE_AND_TRANSLATE_METHODS}. Received: method={method}"
+        )
+    if method in ("linear", "bilinear", "trilinear", "triangle"):
+        method = "linear"
+    elif method in ("cubic", "bicubic", "tricubic"):
+        method = "cubic"
+
+    images = convert_to_tensor(images)
+    scale = convert_to_tensor(scale)
+    translation = convert_to_tensor(translation)
+    kernel = _kernels[method]
+    dtype = backend.result_type(scale.dtype, translation.dtype)
+    scale = scale.astype(dtype)
+    translation = translation.astype(dtype)
+    return _scale_and_translate(
+        images,
+        output_shape,
+        spatial_dims,
+        scale,
+        translation,
+        kernel,
+        antialias,
+    )
