@@ -1,6 +1,12 @@
-def cholesky(a):
+def cholesky(a, upper=False):
     raise NotImplementedError(
-        "`cholesky` is not supported with openvino backend"
+        "`cholesky` is not supported with openvino backend."
+    )
+
+
+def cholesky_inverse(a, upper=False):
+    raise NotImplementedError(
+        "`cholesky_inverse` is not supported with openvino backend."
     )
 
 
@@ -50,3 +56,7 @@ def svd(x, full_matrices=True, compute_uv=True):
 
 def lstsq(a, b, rcond=None):
     raise NotImplementedError("`lstsq` is not supported with openvino backend")
+
+
+def jvp(fun, primals, tangents, has_aux=False):
+    raise NotImplementedError("`jvp` is not supported with openvino backend")
