@@ -1,4 +1,4 @@
-import openvino.runtime.opset14 as ov_opset
+import openvino.opset14 as ov_opset
 from openvino import Type
 
 from keras.src import backend
@@ -486,3 +486,23 @@ def ctc_decode(
 
 def psnr(x1, x2, max_val):
     raise NotImplementedError("`psnr` is not supported with openvino backend")
+
+
+def dot_product_attention(
+    query,
+    key,
+    value,
+    bias=None,
+    mask=None,
+    scale=None,
+    is_causal=False,
+    flash_attention=None,
+    attn_logits_soft_cap=None,
+):
+    raise NotImplementedError(
+        "`dot_product_attention` is not supported with openvino backend"
+    )
+
+
+def unfold(input, kernel_size, dilation=1, padding=0, stride=1):
+    raise NotImplementedError("`unfold` is not supported with openvino backend")
