@@ -1,5 +1,5 @@
 import numpy as np
-import openvino.runtime.opset14 as ov_opset
+import openvino.opset14 as ov_opset
 from openvino import Type
 
 from keras.src.backend import config
@@ -46,6 +46,8 @@ def subtract(x1, x2):
     x1 = get_ov_output(x1, element_type)
     x2 = get_ov_output(x2, element_type)
     x1, x2 = _align_operand_types(x1, x2, "subtract()")
+    if x1.get_element_type() == Type.boolean:
+        return OpenVINOKerasTensor(ov_opset.logical_xor(x1, x2).output(0))
     return OpenVINOKerasTensor(ov_opset.subtract(x1, x2).output(0))
 
 
@@ -74,25 +76,81 @@ def multiply(x1, x2):
 
 
 def mean(x, axis=None, keepdims=False):
-    x = get_ov_output(x)
-    if axis is None:
-        flatten_shape = ov_opset.constant([-1], Type.i32).output(0)
-        x = ov_opset.reshape(x, flatten_shape, False).output(0)
-        axis = 0
-    axis_const = ov_opset.constant(axis, dtype=Type.i32).output(0)
-    mean_ops = ov_opset.reduce_mean(x, axis_const, keepdims)
-    return OpenVINOKerasTensor(mean_ops.output(0))
+    x_ov = get_ov_output(x)
+    x_shape = x_ov.get_partial_shape().to_shape()
+    x_type = x_ov.get_element_type()
+
+    was_axis_none = axis is None
+    x_resolved, axis_resolved = _resolve_axis(x_ov, axis)
+
+    if axis_resolved is None:
+        return OpenVINOKerasTensor(x_ov)
+
+    if x_type.is_integral():
+        ov_type = OPENVINO_DTYPES[config.floatx()]
+        x_resolved = ov_opset.convert(x_resolved, ov_type).output(0)
+
+    result = ov_opset.reduce_mean(x_resolved, axis_resolved, keepdims).output(0)
+
+    if keepdims and was_axis_none:
+        result_shape = [1] * len(x_shape)
+        result = ov_opset.reshape(
+            result,
+            ov_opset.constant(result_shape, Type.i32).output(0),
+            False,
+        ).output(0)
+
+    return OpenVINOKerasTensor(result)
 
 
 def max(x, axis=None, keepdims=False, initial=None):
-    assert initial is None, (
-        "`max` with not None initial is not supported by openvino backend"
-    )
+    return _compute_extrema(x, "max", axis, keepdims, initial)
+
+
+def _compute_extrema(x, operation, axis=None, keepdims=False, initial=None):
+    if operation == "min":
+        reduction_op = ov_opset.reduce_min
+        elementwise_op = ov_opset.minimum
+    elif operation == "max":
+        reduction_op = ov_opset.reduce_max
+        elementwise_op = ov_opset.maximum
+    else:
+        raise ValueError(
+            f"Operation must be 'min' or 'max', received {operation}"
+        )
+
     x = get_ov_output(x)
-    reduce_axis = ov_opset.constant(axis, Type.i32).output(0)
-    return OpenVINOKerasTensor(
-        ov_opset.reduce_max(x, reduce_axis, keepdims).output(0)
-    )
+    x_type = x.get_element_type()
+    x_for_rank = x
+
+    is_bool = x_type == Type.boolean
+    if is_bool:
+        x = ov_opset.convert(x, Type.i32).output(0)
+        x_type = Type.i32
+
+    if isinstance(axis, tuple) and len(axis) == 0:
+        return OpenVINOKerasTensor(x)
+
+    was_axis_none = axis is None
+    x, axis = _resolve_axis(x, axis)
+
+    result = reduction_op(x, axis, keepdims).output(0)
+
+    if initial is not None:
+        initial_tensor = ov_opset.constant(initial, x_type).output(0)
+        result = elementwise_op(result, initial_tensor).output(0)
+
+    if keepdims and was_axis_none:
+        orig_shape = ov_opset.shape_of(x_for_rank, Type.i32).output(0)
+        orig_rank_shape = ov_opset.shape_of(orig_shape, Type.i32).output(0)
+        one = ov_opset.constant(1, Type.i32).output(0)
+        result_shape = ov_opset.broadcast(one, orig_rank_shape).output(0)
+        result = ov_opset.reshape(result, result_shape, False).output(0)
+
+    if is_bool:
+        result = ov_opset.convert(result, Type.boolean).output(0)
+
+    return OpenVINOKerasTensor(result)
 
 
 def ones(shape, dtype=None):
@@ -123,6 +181,9 @@ def zeros(shape, dtype=None):
 
 def absolute(x):
     x = get_ov_output(x)
+    x_type = x.get_element_type()
+    if x_type == Type.boolean:
+        return OpenVINOKerasTensor(x)
     return OpenVINOKerasTensor(ov_opset.absolute(x).output(0))
 
 
@@ -133,11 +194,10 @@ def abs(x):
 
 def all(x, axis=None, keepdims=False):
     x = get_ov_output(x)
+    x, axis = _resolve_axis(x, axis)
     if axis is None:
-        flatten_shape = ov_opset.constant([-1], Type.i32).output(0)
-        x = ov_opset.reshape(x, flatten_shape, False).output(0)
-        axis = 0
-    axis = ov_opset.constant(axis, Type.i32).output(0)
+        return OpenVINOKerasTensor(x)
+    x = ov_opset.convert(x, Type.boolean).output(0)
     return OpenVINOKerasTensor(
         ov_opset.reduce_logical_and(x, axis, keepdims).output(0)
     )
@@ -149,28 +209,21 @@ def angle(x):
 
 def any(x, axis=None, keepdims=False):
     x = get_ov_output(x)
+    x, axis = _resolve_axis(x, axis)
     if axis is None:
-        flatten_shape = ov_opset.constant([-1], Type.i32).output(0)
-        x = ov_opset.reshape(x, flatten_shape, False).output(0)
-        axis = 0
-    axis = ov_opset.constant(axis, Type.i32).output(0)
+        return OpenVINOKerasTensor(x)
+    x = ov_opset.convert(x, Type.boolean).output(0)
     return OpenVINOKerasTensor(
         ov_opset.reduce_logical_or(x, axis, keepdims).output(0)
     )
 
 
 def amax(x, axis=None, keepdims=False):
-    if axis == () or axis == []:
-        return x
     x = get_ov_output(x)
     x_type = x.get_element_type()
+    x, axis = _resolve_axis(x, axis)
     if axis is None:
-        flatten_shape = ov_opset.constant([-1], Type.i32).output(0)
-        x = ov_opset.reshape(x, flatten_shape, False).output(0)
-        axis = 0
-    if isinstance(axis, tuple):
-        axis = list(axis)
-    axis = ov_opset.constant(axis, Type.i32).output(0)
+        return OpenVINOKerasTensor(x)
     if x_type == Type.boolean:
         return OpenVINOKerasTensor(
             ov_opset.reduce_logical_or(x, axis, keepdims).output(0)
@@ -179,10 +232,21 @@ def amax(x, axis=None, keepdims=False):
 
 
 def amin(x, axis=None, keepdims=False):
-    if axis == () or axis == []:
-        return x
     x = get_ov_output(x)
     x_type = x.get_element_type()
+    x, axis = _resolve_axis(x, axis)
+    if axis is None:
+        return OpenVINOKerasTensor(x)
+    if x_type == Type.boolean:
+        return OpenVINOKerasTensor(
+            ov_opset.reduce_logical_and(x, axis, keepdims).output(0)
+        )
+    return OpenVINOKerasTensor(ov_opset.reduce_min(x, axis, keepdims).output(0))
+
+
+def _resolve_axis(x, axis):
+    if axis == () or axis == []:
+        return x, None
     if axis is None:
         flatten_shape = ov_opset.constant([-1], Type.i32).output(0)
         x = ov_opset.reshape(x, flatten_shape, False).output(0)
@@ -190,11 +254,18 @@ def amin(x, axis=None, keepdims=False):
     if isinstance(axis, tuple):
         axis = list(axis)
     axis = ov_opset.constant(axis, Type.i32).output(0)
+    return x, axis
+
+
+def _upcast_type_if_needed(x):
+    x_type = x.get_element_type()
     if x_type == Type.boolean:
-        return OpenVINOKerasTensor(
-            ov_opset.reduce_logical_and(x, axis, keepdims).output(0)
-        )
-    return OpenVINOKerasTensor(ov_opset.reduce_min(x, axis, keepdims).output(0))
+        x = ov_opset.convert(x, Type.i32).output(0)
+    elif x_type in (Type.i8, Type.i16):
+        x = ov_opset.convert(x, Type.i32).output(0)
+    elif x_type in (Type.u8, Type.u16):
+        x = ov_opset.convert(x, Type.u32).output(0)
+    return x
 
 
 def append(x1, x2, axis=None):
@@ -208,7 +279,7 @@ def append(x1, x2, axis=None):
     return OpenVINOKerasTensor(ov_opset.concat([x1, x2], axis).output(0))
 
 
-def arange(start, stop=None, step=1, dtype=None):
+def arange(start, stop=None, step=None, dtype=None):
     if stop is None:
         start, stop = get_ov_output(0), get_ov_output(start)
     else:
@@ -437,6 +508,10 @@ def array(x, dtype=None):
     return np.array(x)
 
 
+def view(x, dtype=None):
+    raise NotImplementedError("`view` is not supported with openvino backend")
+
+
 def average(x, axis=None, weights=None):
     x = get_ov_output(x)
     if weights is not None:
@@ -557,11 +632,18 @@ def cbrt(x):
 
 def ceil(x):
     x = get_ov_output(x)
-    return OpenVINOKerasTensor(ov_opset.ceil(x).output(0))
+    x_type = x.get_element_type()
+    if x_type.is_integral():
+        x = ov_opset.convert(x, OPENVINO_DTYPES[config.floatx()]).output(0)
+    ceiling = ov_opset.ceil(x).output(0)
+    return OpenVINOKerasTensor(ceiling)
 
 
 def clip(x, x_min, x_max):
     x = get_ov_output(x)
+    x_type = x.get_element_type()
+    if x_type == Type.boolean:
+        x = ov_opset.convert(x, Type.i32).output(0)
     x_min = get_ov_output(x_min, x.get_element_type())
     x_max = get_ov_output(x_max, x.get_element_type())
     clip_by_min = ov_opset.maximum(x, x_min).output(0)
@@ -649,13 +731,30 @@ def cumsum(x, axis=None, dtype=None):
         x = ov_opset.reshape(x, flatten_shape, False).output(0)
         axis = 0
     axis = ov_opset.constant(axis, Type.i32).output(0)
+    if x.get_element_type() == Type.boolean:
+        x = ov_opset.convert(x, Type.i32).output(0)
     return OpenVINOKerasTensor(ov_opset.cumsum(x, axis).output(0))
 
 
 def deg2rad(x):
-    raise NotImplementedError(
-        "`deg2rad` is not supported with openvino backend"
-    )
+    x = get_ov_output(x)
+    x_type = x.get_element_type()
+    pi_over_180 = np.pi / 180.0
+
+    if x_type == Type.i64:
+        output_type = Type.f64
+    elif x_type.is_integral():
+        output_type = OPENVINO_DTYPES[config.floatx()]
+    else:
+        output_type = x_type
+
+    if x_type != output_type:
+        x = ov_opset.convert(x, output_type)
+
+    const_pi_over_180 = ov_opset.constant(pi_over_180, output_type).output(0)
+    result = ov_opset.multiply(x, const_pi_over_180).output(0)
+
+    return OpenVINOKerasTensor(result)
 
 
 def diag(x, k=0):
@@ -672,7 +771,7 @@ def diff(a, n=1, axis=-1):
     if n == 0:
         return OpenVINOKerasTensor(get_ov_output(a))
     if n < 0:
-        raise ValueError("order must be non-negative but got " + repr(n))
+        raise ValueError(f"order must be non-negative but got {repr(n)}")
     a = get_ov_output(a)
     a_type = a.get_element_type()
     if isinstance(a, np.ndarray):
@@ -821,6 +920,9 @@ def flip(x, axis=None):
 
 def floor(x):
     x = get_ov_output(x)
+    x_type = x.get_element_type()
+    if x_type.is_integral():
+        x = ov_opset.convert(x, OPENVINO_DTYPES[config.floatx()])
     return OpenVINOKerasTensor(ov_opset.floor(x).output(0))
 
 
@@ -848,6 +950,10 @@ def full_like(x, fill_value, dtype=None):
     return OpenVINOKerasTensor(res)
 
 
+def gcd(x1, x2):
+    raise NotImplementedError("`gcd` is not supported with openvino backend")
+
+
 def greater(x1, x2):
     element_type = None
     if isinstance(x1, OpenVINOKerasTensor):
@@ -887,6 +993,10 @@ def hstack(xs):
     return OpenVINOKerasTensor(ov_opset.concat(elems, axis).output(0))
 
 
+def hypot(x1, x2):
+    raise NotImplementedError("`hypot` is not supported with openvino backend")
+
+
 def identity(n, dtype=None):
     n = get_ov_output(n)
     dtype = Type.f32 if dtype is None else dtype
@@ -925,20 +1035,91 @@ def isclose(x1, x2, rtol=1e-5, atol=1e-8, equal_nan=False):
 
 
 def isfinite(x):
-    x = get_ov_output(x)
-    return OpenVINOKerasTensor(ov_opset.is_finite(x).output(0))
+    # NOTE: openvino has an is_finite operation but it does not properly
+    # catch np.inf and -np.inf as not finite values. Hence we bootstrap here. If
+    # that ever changes, we could simplify this to just call that operation.
+    inf_values = get_ov_output(isinf(x))
+    nan_values = get_ov_output(isnan(x))
+    all_non_finite_values = ov_opset.logical_or(inf_values, nan_values).output(
+        0
+    )
+    is_finite = ov_opset.logical_not(all_non_finite_values).output(0)
+    return OpenVINOKerasTensor(is_finite)
+
+
+def isin(x1, x2, assume_unique=False, invert=False):
+    raise NotImplementedError("`isin` is not supported with openvino backend")
 
 
 def isinf(x):
-    x = get_ov_output(x)
-    return OpenVINOKerasTensor(ov_opset.is_inf(x).output(0))
+    pos_inf = get_ov_output(isposinf(x))
+    neg_inf = get_ov_output(isneginf(x))
+    inf = ov_opset.logical_or(pos_inf, neg_inf).output(0)
+    return OpenVINOKerasTensor(inf)
 
 
 def isnan(x):
     x = get_ov_output(x)
+    x_type = x.get_element_type()
+    if x_type.is_integral():
+        x = ov_opset.convert(x, OPENVINO_DTYPES[config.floatx()])
     return OpenVINOKerasTensor(ov_opset.is_nan(x).output(0))
 
 
+def isneginf(x):
+    return _is_inf(x, pos=False)
+
+
+def isposinf(x):
+    return _is_inf(x)
+
+
+def _is_inf(x, pos=True):
+    # NOTE: there is an ov_opset.is_inf but it does not catch
+    # numpy infinite values like np.inf and -np.inf, hence why we have this
+    # if this ever changes in OpenVINO, we can do this instead:
+    # ov_opset.is_inf(x, {"detect_positive": pos, "detect_negative": not pos})
+    # for each infinite sign
+    inf_value = np.inf if pos else -np.inf
+    x = get_ov_output(x)
+    x_type = x.get_element_type()
+
+    if x_type.is_integral() or x_type == Type.boolean:
+        shape = ov_opset.shape_of(x, "i32").output(0)
+        false_const = ov_opset.constant(False, Type.boolean).output(0)
+        return OpenVINOKerasTensor(
+            ov_opset.broadcast(false_const, shape).output(0)
+        )
+
+    if x_type == Type.bf16:
+        x_f32 = ov_opset.convert(x, Type.f32).output(0)
+        inf = ov_opset.constant(inf_value, Type.f32).output(0)
+        is_inf = ov_opset.equal(x_f32, inf).output(0)
+    else:
+        if x_type == Type.f16:
+            inf = ov_opset.constant(inf_value, Type.f16).output(0)
+        elif x_type == Type.f32:
+            inf = ov_opset.constant(inf_value, Type.f32).output(0)
+        elif x_type == Type.f64:
+            inf = ov_opset.constant(inf_value, Type.f64).output(0)
+        else:
+            inf = ov_opset.constant(inf_value, Type.f32).output(0)
+        is_inf = ov_opset.equal(x, inf).output(0)
+    return OpenVINOKerasTensor(is_inf)
+
+
+def isreal(x):
+    raise NotImplementedError("`isreal` is not supported with openvino backend")
+
+
+def kron(x1, x2):
+    raise NotImplementedError("`kron` is not supported with openvino backend")
+
+
+def lcm(x1, x2):
+    raise NotImplementedError("`lcm` is not supported with openvino backend")
+
+
 def less(x1, x2):
     element_type = None
     if isinstance(x1, OpenVINOKerasTensor):
@@ -966,9 +1147,131 @@ def less_equal(x1, x2):
 def linspace(
     start, stop, num=50, endpoint=True, retstep=False, dtype=None, axis=0
 ):
-    raise NotImplementedError(
-        "`linspace` is not supported with openvino backend"
+    """Return evenly spaced numbers over a specified interval.
+
+    Supports axis=0 (prepend) and axis=-1 (append). Intermediate axis values are
+    treated as axis=-1.
+
+    If `retstep` is True, also returns the step size between values.
+
+    """
+
+    start = get_ov_output(start)
+    stop = get_ov_output(stop)
+
+    if hasattr(num, "output") or isinstance(num, OpenVINOKerasTensor):
+        num_tensor = get_ov_output(num)
+        try:
+            if num_tensor.get_node().get_type_name() == "Constant":
+                num_value = num_tensor.get_node().get_vector()[0]
+                num = int(num_value)
+            else:
+                raise NotImplementedError(
+                    "Dynamic num values not fully supported"
+                )
+        except Exception as e:
+            raise NotImplementedError(
+                "Could not extract num value from tensor"
+            ) from e
+    else:
+        num = int(num)
+
+    if dtype is None:
+        output_type = OPENVINO_DTYPES[config.floatx()]
+    else:
+        output_type = OPENVINO_DTYPES[dtype]
+
+    start = ov_opset.convert(start, output_type).output(0)
+    stop = ov_opset.convert(stop, output_type).output(0)
+
+    if num < 0:
+        raise ValueError("Number of samples, `num`, must be non-negative.")
+
+    if num == 0:
+        empty_shape = ov_opset.constant([0], Type.i32).output(0)
+        result = ov_opset.broadcast(
+            ov_opset.constant(0.0, output_type).output(0), empty_shape
+        ).output(0)
+        if retstep:
+            nan_step = ov_opset.constant(np.nan, output_type).output(0)
+            return OpenVINOKerasTensor(result), OpenVINOKerasTensor(nan_step)
+        return OpenVINOKerasTensor(result)
+
+    if num == 1:
+        result_val = start
+        axis_const = ov_opset.constant([axis], Type.i32).output(0)
+        result = ov_opset.unsqueeze(result_val, axis_const).output(0)
+        if retstep:
+            if endpoint:
+                step = ov_opset.constant(np.nan, output_type).output(0)
+            else:
+                step = ov_opset.subtract(stop, start).output(0)
+            return OpenVINOKerasTensor(result), OpenVINOKerasTensor(step)
+    zero_i32 = ov_opset.constant(0, Type.i32).output(0)
+    one_i32 = ov_opset.constant(1, Type.i32).output(0)
+    one_i32_array = ov_opset.constant([1], Type.i32).output(0)
+
+    num_const = ov_opset.constant(num, output_type).output(0)
+
+    if endpoint:
+        divisor = ov_opset.subtract(
+            num_const, ov_opset.constant(1, output_type).output(0)
+        ).output(0)
+    else:
+        divisor = num_const
+
+    step = ov_opset.divide(
+        ov_opset.subtract(stop, start).output(0), divisor
+    ).output(0)
+
+    indices = ov_opset.range(
+        zero_i32,
+        ov_opset.constant(num, Type.i32).output(0),
+        one_i32,
+        output_type,
+    ).output(0)
+
+    start_shape = ov_opset.convert(
+        ov_opset.shape_of(start).output(0), Type.i32
+    ).output(0)
+    indices_shape = ov_opset.convert(
+        ov_opset.shape_of(indices).output(0), Type.i32
+    ).output(0)
+
+    start_rank = ov_opset.shape_of(start_shape).output(0)
+    ones_for_start = ov_opset.broadcast(one_i32, start_rank).output(0)
+
+    if axis == 0:
+        indices_target_shape = ov_opset.concat(
+            [indices_shape, ones_for_start], 0
+        ).output(0)
+        start_target_shape = ov_opset.concat(
+            [one_i32_array, start_shape], 0
+        ).output(0)
+    else:
+        indices_target_shape = ov_opset.concat(
+            [ones_for_start, indices_shape], 0
+        ).output(0)
+        start_target_shape = ov_opset.concat(
+            [start_shape, one_i32_array], 0
+        ).output(0)
+
+    indices_reshaped = ov_opset.reshape(
+        indices, indices_target_shape, False
+    ).output(0)
+    start_reshaped = ov_opset.reshape(start, start_target_shape, False).output(
+        0
     )
+    step_reshaped = ov_opset.reshape(step, start_target_shape, False).output(0)
+
+    scaled_indices = ov_opset.multiply(indices_reshaped, step_reshaped).output(
+        0
+    )
+    result = ov_opset.add(start_reshaped, scaled_indices).output(0)
+
+    if retstep:
+        return OpenVINOKerasTensor(result), OpenVINOKerasTensor(step)
+    return OpenVINOKerasTensor(result)
 
 
 def log(x):
@@ -1021,8 +1324,58 @@ def log2(x):
 
 
 def logaddexp(x1, x2):
+    element_type = None
+    if isinstance(x1, OpenVINOKerasTensor):
+        element_type = x1.output.get_element_type()
+    if isinstance(x2, OpenVINOKerasTensor):
+        element_type = x2.output.get_element_type()
+    x1 = get_ov_output(x1, element_type)
+    x2 = get_ov_output(x2, element_type)
+    x1, x2 = _align_operand_types(x1, x2, "logaddexp()")
+
+    if x1.element_type.is_integral() or x2.element_type.is_integral():
+        float_dtype = OPENVINO_DTYPES[config.floatx()]
+        if x1.element_type.is_integral():
+            x1 = ov_opset.convert(x1, float_dtype)
+        if x2.element_type.is_integral():
+            x2 = ov_opset.convert(x2, float_dtype)
+
+    # Get the output nodes properly
+    max_val_node = ov_opset.maximum(x1, x2)
+    max_val = max_val_node.output(0)
+
+    # Compute absolute difference
+    sub_node = ov_opset.subtract(x1, x2)
+    abs_diff_node = ov_opset.abs(sub_node.output(0))
+    abs_diff = abs_diff_node.output(0)
+
+    # Compute negative absolute difference and its exponential
+    neg_abs_diff_node = ov_opset.negative(abs_diff)
+    neg_abs_diff = neg_abs_diff_node.output(0)
+    exp_neg_abs_node = ov_opset.exp(neg_abs_diff)
+    exp_neg_abs = exp_neg_abs_node.output(0)
+
+    # Get the element type from the node, not the output
+    element_type = exp_neg_abs_node.get_element_type()
+    one_node = ov_opset.constant(1, element_type)
+    one = one_node.output(0)
+
+    # Compute log term
+    one_plus_exp_node = ov_opset.add(one, exp_neg_abs)
+    one_plus_exp = one_plus_exp_node.output(0)
+    log_term_node = ov_opset.log(one_plus_exp)
+    log_term = log_term_node.output(0)
+
+    # Final result
+    result_node = ov_opset.add(max_val, log_term)
+    result = result_node.output(0)
+
+    return OpenVINOKerasTensor(result)
+
+
+def logaddexp2(x1, x2):
     raise NotImplementedError(
-        "`logaddexp` is not supported with openvino backend"
+        "`logaddexp2` is not supported with openvino backend"
     )
 
 
@@ -1049,10 +1402,30 @@ def logical_or(x1, x2):
 
 
 def logspace(start, stop, num=50, endpoint=True, base=10, dtype=None, axis=0):
-    raise NotImplementedError(
-        "`logspace` is not supported with openvino backend"
+    linear_samples = linspace(
+        start=start,
+        stop=stop,
+        num=num,
+        endpoint=endpoint,
+        retstep=False,
+        dtype=dtype,
+        axis=axis,
     )
 
+    if dtype is None:
+        output_type = OPENVINO_DTYPES[config.floatx()]
+    else:
+        output_type = OPENVINO_DTYPES[dtype]
+
+    linear_output = get_ov_output(linear_samples)
+    base_tensor = get_ov_output(base)
+
+    base_tensor = ov_opset.convert(base_tensor, output_type).output(0)
+
+    result = ov_opset.power(base_tensor, linear_output).output(0)
+
+    return OpenVINOKerasTensor(result)
+
 
 def maximum(x1, x2):
     x1 = get_ov_output(x1)
@@ -1062,56 +1435,186 @@ def maximum(x1, x2):
 
 
 def median(x, axis=None, keepdims=False):
-    raise NotImplementedError("`median` is not supported with openvino backend")
+    x = get_ov_output(x)
+    x_shape = x.get_partial_shape()
+    rank = x_shape.rank.get_length()
+
+    if rank == 0:
+        return OpenVINOKerasTensor(x)
+
+    # Handle axis=None by flattening the input
+    flattened_all = False
+    if axis is None:
+        x = ov_opset.reshape(x, [-1], False).output(0)
+        axis = 0
+        original_rank = rank
+        rank = 1
+        flattened_all = True
+    else:
+        # Handle tuple axis - for median, we only support single axis
+        if isinstance(axis, (tuple, list)):
+            if len(axis) != 1:
+                raise ValueError("median only supports single axis reduction")
+            axis = axis[0]
+
+        # Handle negative axis
+        if axis < 0:
+            axis = rank + axis
+        original_rank = rank
+
+    # Get the size of the dimension to sort
+    shape_tensor = ov_opset.shape_of(x, output_type=Type.i32).output(0)
+    k = ov_opset.gather(
+        shape_tensor,
+        ov_opset.constant([axis], Type.i32).output(0),
+        ov_opset.constant(0, Type.i32).output(0),
+    ).output(0)
+
+    # Convert k to a scalar value
+    k_scalar = ov_opset.squeeze(k, [0]).output(0)
+
+    # Use topk with k=size_of_axis to get all elements sorted
+    topk_outputs = ov_opset.topk(
+        x, k=k_scalar, axis=axis, mode="min", sort="value", stable=True
+    )
+
+    # Get the sorted values
+    sorted_values = topk_outputs.output(0)
+
+    # Convert to float for median calculation
+    x1_type = ov_to_keras_type(sorted_values.get_element_type())
+    result_type = dtypes.result_type(x1_type, float)
+    result_type = OPENVINO_DTYPES[result_type]
+    sorted_values = ov_opset.convert(sorted_values, result_type).output(0)
+
+    # Calculate median indices
+    # For odd length: median_idx = (k-1) // 2
+    # For even length: we need indices (k//2 - 1) and k//2, then average
+
+    k_minus_1 = ov_opset.subtract(
+        k_scalar, ov_opset.constant(1, Type.i32).output(0)
+    ).output(0)
+    k_div_2 = ov_opset.divide(
+        k_scalar, ov_opset.constant(2, Type.i32).output(0)
+    ).output(0)
+    k_minus_1_div_2 = ov_opset.divide(
+        k_minus_1, ov_opset.constant(2, Type.i32).output(0)
+    ).output(0)
+
+    # Check if k is odd
+    k_mod_2 = ov_opset.mod(
+        k_scalar, ov_opset.constant(2, Type.i32).output(0)
+    ).output(0)
+    is_odd = ov_opset.equal(
+        k_mod_2, ov_opset.constant(1, Type.i32).output(0)
+    ).output(0)
+
+    # For odd case: take the middle element
+    odd_idx = k_minus_1_div_2
+
+    # For even case: take average of two middle elements
+    even_idx1 = ov_opset.subtract(
+        k_div_2, ov_opset.constant(1, Type.i32).output(0)
+    ).output(0)
+    even_idx2 = k_div_2
+
+    # Gather elements for both cases
+    # Create gather indices tensor for the axis
+    gather_indices_odd = ov_opset.unsqueeze(odd_idx, [0]).output(0)
+    gather_indices_even1 = ov_opset.unsqueeze(even_idx1, [0]).output(0)
+    gather_indices_even2 = ov_opset.unsqueeze(even_idx2, [0]).output(0)
+
+    # Gather the median elements
+    odd_result = ov_opset.gather(
+        sorted_values,
+        gather_indices_odd,
+        ov_opset.constant(axis, Type.i32).output(0),
+    ).output(0)
+    even_result1 = ov_opset.gather(
+        sorted_values,
+        gather_indices_even1,
+        ov_opset.constant(axis, Type.i32).output(0),
+    ).output(0)
+    even_result2 = ov_opset.gather(
+        sorted_values,
+        gather_indices_even2,
+        ov_opset.constant(axis, Type.i32).output(0),
+    ).output(0)
+
+    # Average the two middle elements for even case
+    even_sum = ov_opset.add(even_result1, even_result2).output(0)
+    even_result = ov_opset.divide(
+        even_sum, ov_opset.constant(2.0, result_type).output(0)
+    ).output(0)
+
+    # Select between odd and even results
+    median_result = ov_opset.select(is_odd, odd_result, even_result).output(0)
+
+    # Remove the gathered dimension (squeeze)
+    median_result = ov_opset.squeeze(median_result, [axis]).output(0)
+
+    # Handle keepdims
+    if keepdims:
+        if flattened_all:
+            # When axis=None, keepdims should restore all dimensions as 1
+            ones_shape = ov_opset.constant(
+                [1] * original_rank, Type.i32
+            ).output(0)
+            median_result = ov_opset.reshape(
+                median_result, ones_shape, False
+            ).output(0)
+        else:
+            median_result = ov_opset.unsqueeze(median_result, [axis]).output(0)
+
+    return OpenVINOKerasTensor(median_result)
 
 
 def meshgrid(*x, indexing="xy"):
-    raise NotImplementedError(
-        "`meshgrid` is not supported with openvino backend"
-    )
+    if len(x) < 2:
+        raise ValueError(
+            "meshgrid requires at least 2 input arrays. "
+            f"Received: {len(x)} input array(s)."
+        )
+    if indexing not in ("xy", "ij"):
+        raise ValueError("indexing must be either 'xy' or 'ij'")
+
+    tensors = [get_ov_output(xi) for xi in x]
+    n = len(tensors)
+
+    shapes = [
+        ov_opset.shape_of(t, Type.i64).output(0) for t in tensors
+    ]  # each is [Ni]
+    one = ov_opset.constant([1], Type.i64).output(0)
+
+    if indexing == "xy":
+        shape_list = [shapes[1], shapes[0]] + shapes[2:]
+        out_shape = ov_opset.concat(shape_list, axis=0).output(0)
+    else:
+        out_shape = ov_opset.concat(shapes, axis=0).output(0)
+
+    outputs = []
+    for i, t in enumerate(tensors):
+        reshape_parts = [one] * n
+        if indexing == "xy":
+            if i == 0:
+                reshape_parts[1] = shapes[0]
+            elif i == 1:
+                reshape_parts[0] = shapes[1]
+            else:
+                reshape_parts[i] = shapes[i]
+        else:
+            reshape_parts[i] = shapes[i]
+
+        reshape_shape = ov_opset.concat(reshape_parts, axis=0).output(0)
+        reshaped = ov_opset.reshape(t, reshape_shape, False).output(0)
+        broadcasted = ov_opset.broadcast(reshaped, out_shape).output(0)
+        outputs.append(OpenVINOKerasTensor(broadcasted))
+
+    return outputs
 
 
 def min(x, axis=None, keepdims=False, initial=None):
-    x = get_ov_output(x)
-    original_type = x.get_element_type()
-    x_type = original_type
-    x_shape = x.get_partial_shape().to_shape()
-
-    is_bool = x_type == Type.boolean
-    if is_bool:
-        x = ov_opset.convert(x, Type.i32).output(0)
-        x_type = Type.i32
-
-    if isinstance(axis, tuple) and len(axis) == 0:
-        return OpenVINOKerasTensor(x)
-
-    if axis is None:
-        flatten_shape = ov_opset.constant([-1], Type.i32).output(0)
-        x = ov_opset.reshape(x, flatten_shape, False).output(0)
-        axis = 0
-
-    if isinstance(axis, tuple):
-        axis = list(axis)
-
-    axis_const = ov_opset.constant(axis, Type.i32).output(0)
-    min_result = ov_opset.reduce_min(x, axis_const, keepdims).output(0)
-
-    if initial is not None:
-        initial_tensor = ov_opset.constant(initial, x_type).output(0)
-        min_result = ov_opset.minimum(min_result, initial_tensor).output(0)
-
-    if keepdims:
-        result_shape = [1] * len(x_shape)
-        min_result = ov_opset.reshape(
-            min_result,
-            ov_opset.constant(result_shape, Type.i32).output(0),
-            False,
-        ).output(0)
-
-    if is_bool:
-        min_result = ov_opset.convert(min_result, Type.boolean).output(0)
-
-    return OpenVINOKerasTensor(min_result)
+    return _compute_extrema(x, "min", axis, keepdims, initial)
 
 
 def minimum(x1, x2):
@@ -1278,7 +1781,22 @@ def pad(x, pad_width, mode="constant", constant_values=None):
 
 
 def prod(x, axis=None, keepdims=False, dtype=None):
-    raise NotImplementedError("`prod` is not supported with openvino backend")
+    x = get_ov_output(x)
+
+    # If a specific dtype is requested, cast the input to that dtype.
+    if dtype is not None:
+        ov_dtype = OPENVINO_DTYPES[standardize_dtype(dtype)]
+        x = ov_opset.convert(x, ov_dtype).output(0)
+    # Otherwise, apply dtype promotion rules before reduction.
+    else:
+        x = _upcast_type_if_needed(x)
+    x, axis = _resolve_axis(x, axis)
+    if axis is None:
+        return OpenVINOKerasTensor(x)
+    # Compute the product
+    result = ov_opset.reduce_prod(x, axis, keepdims).output(0)
+
+    return OpenVINOKerasTensor(result)
 
 
 def quantile(x, q, axis=None, method="linear", keepdims=False):
@@ -1307,7 +1825,62 @@ def reciprocal(x):
 
 
 def repeat(x, repeats, axis=None):
-    raise NotImplementedError("`repeat` is not supported with openvino backend")
+    x = get_ov_output(x)
+    const_0 = ov_opset.constant(0, Type.i32)
+    const_1 = ov_opset.constant(1, Type.i32)
+    const_neg_1 = ov_opset.constant([-1], Type.i32)
+
+    if axis is not None and axis < 0:
+        axis += len(x.get_partial_shape())
+
+    if axis is None:
+        x = ov_opset.reshape(x, const_neg_1, special_zero=False)
+        axis = 0
+
+    if isinstance(repeats, (int, np.integer)) or (
+        isinstance(repeats, np.ndarray)
+        and repeats.ndim == 1
+        and repeats.size == 1
+    ):
+        repeats_val = (
+            int(repeats)
+            if isinstance(repeats, (np.integer, np.ndarray))
+            else repeats
+        )
+        dim_len = ov_opset.gather(
+            ov_opset.shape_of(x, Type.i32),
+            ov_opset.constant([axis], Type.i32),
+            const_0,
+        )
+        dim_len = ov_opset.squeeze(dim_len, ov_opset.constant([0], Type.i32))
+        idx_range = ov_opset.range(
+            const_0, dim_len, const_1, output_type=Type.i32
+        )
+        idx_range = ov_opset.unsqueeze(idx_range, const_1)
+        tiled = ov_opset.tile(
+            idx_range, ov_opset.constant([1, repeats_val], Type.i32)
+        )
+        idx = ov_opset.reshape(tiled, const_neg_1, special_zero=False)
+        result = ov_opset.gather(x, idx, ov_opset.constant(axis, Type.i32))
+        return OpenVINOKerasTensor(result.output(0))
+    repeats_tensor = get_ov_output(repeats)
+    cumsum = ov_opset.cumsum(repeats_tensor, const_0)
+    total = ov_opset.reduce_sum(
+        repeats_tensor, ov_opset.constant([0], Type.i32), keep_dims=False
+    )
+    total = ov_opset.convert(total, Type.i32)
+    out_indices = ov_opset.range(const_0, total, const_1, output_type=Type.i32)
+    cumsum_unsq = ov_opset.unsqueeze(cumsum, const_0)
+    out_indices_unsq = ov_opset.unsqueeze(out_indices, const_1)
+    cumsum_unsq = ov_opset.convert(cumsum_unsq, Type.i32)
+    mask = ov_opset.greater_equal(out_indices_unsq, cumsum_unsq)
+    gather_indices = ov_opset.reduce_sum(
+        ov_opset.convert(mask, Type.i32), ov_opset.constant([1], Type.i32)
+    )
+    result = ov_opset.gather(
+        x, gather_indices, ov_opset.constant(axis, Type.i32)
+    )
+    return OpenVINOKerasTensor(result.output(0))
 
 
 def reshape(x, newshape):
@@ -1356,7 +1929,43 @@ def size(x):
 
 
 def sort(x, axis=-1):
-    raise NotImplementedError("`sort` is not supported with openvino backend")
+    x = get_ov_output(x)
+    x_shape = x.get_partial_shape()
+    rank = x_shape.rank.get_length()
+
+    if rank == 0:
+        return OpenVINOKerasTensor(x)
+
+    # Handle axis=None by flattening the input
+    if axis is None:
+        x = ov_opset.reshape(
+            x, ov_opset.constant([-1], Type.i32), False
+        ).output(0)
+        axis = 0
+    # Handle negative axis
+    elif axis < 0:
+        axis = rank + axis
+
+    # Get the size of the dimension to sort
+    shape_tensor = ov_opset.shape_of(x, output_type=Type.i32).output(0)
+    k = ov_opset.gather(
+        shape_tensor,
+        ov_opset.constant([axis], Type.i32).output(0),
+        ov_opset.constant(0, Type.i32).output(0),
+    ).output(0)
+
+    # Convert k to a scalar value
+    k_scalar = ov_opset.squeeze(k, ov_opset.constant([0], Type.i32)).output(0)
+
+    # Use topk with k=size_of_axis to get all elements sorted
+    topk_outputs = ov_opset.topk(
+        x, k=k_scalar, axis=axis, mode="min", sort="value", stable=True
+    )
+
+    # Get the sorted values
+    sorted_values = topk_outputs.output(0)
+
+    return OpenVINOKerasTensor(sorted_values)
 
 
 def split(x, indices_or_sections, axis=0):
@@ -1744,12 +2353,19 @@ def negative(x):
 
 def square(x):
     x = get_ov_output(x)
+    x_type = x.get_element_type()
+    if x_type == Type.boolean:
+        x = ov_opset.convert(x, Type.i32).output(0)
     const_two = ov_opset.constant(2, x.get_element_type()).output(0)
     return OpenVINOKerasTensor(ov_opset.power(x, const_two).output(0))
 
 
 def sqrt(x):
     x = get_ov_output(x)
+    x_type = x.get_element_type()
+    if x_type.is_integral():
+        ov_type = OPENVINO_DTYPES[config.floatx()]
+        x = ov_opset.convert(x, ov_type).output(0)
     return OpenVINOKerasTensor(ov_opset.sqrt(x).output(0))
 
 
@@ -1760,6 +2376,8 @@ def squeeze(x, axis=None):
         for idx, dim in enumerate(x.get_partial_shape()):
             if dim == 1:
                 axis.append(idx)
+    if isinstance(axis, tuple):
+        axis = list(axis)
     axis = ov_opset.constant(axis, Type.i32).output(0)
     return OpenVINOKerasTensor(ov_opset.squeeze(x, axis).output(0))
 
@@ -1784,6 +2402,12 @@ def transpose(x, axes=None):
     return OpenVINOKerasTensor(ov_opset.transpose(x, axes).output(0))
 
 
+def trapezoid(y, x=None, dx=1.0, axis=-1):
+    raise NotImplementedError(
+        "`trapezoid` is not supported with openvino backend"
+    )
+
+
 def var(x, axis=None, keepdims=False):
     x = get_ov_output(x)
     if axis is None:
@@ -1806,16 +2430,27 @@ def var(x, axis=None, keepdims=False):
 
 def sum(x, axis=None, keepdims=False):
     x = get_ov_output(x)
+    x, axis = _resolve_axis(x, axis)
     if axis is None:
-        flatten_shape = ov_opset.constant([-1], Type.i32).output(0)
-        x = ov_opset.reshape(x, flatten_shape, False).output(0)
-        axis = 0
-    axis = ov_opset.constant(axis, Type.i32).output(0)
-    return OpenVINOKerasTensor(ov_opset.reduce_sum(x, axis, keepdims).output(0))
+        return OpenVINOKerasTensor(x)
+    x = _upcast_type_if_needed(x)
+    summed_value = ov_opset.reduce_sum(x, axis, keepdims).output(0)
+    return OpenVINOKerasTensor(summed_value)
 
 
 def eye(N, M=None, k=0, dtype=None):
-    raise NotImplementedError("`eye` is not supported with openvino backend")
+    dtype = standardize_dtype(dtype) or config.floatx()
+    ov_type = OPENVINO_DTYPES[dtype]
+    if M is None:
+        M = N
+    return OpenVINOKerasTensor(
+        ov_opset.eye(
+            ov_opset.constant(N, Type.i32),
+            ov_opset.constant(M, Type.i32),
+            ov_opset.constant(k, Type.i32),
+            output_type=ov_type,
+        ).output(0)
+    )
 
 
 def floor_divide(x1, x2):
