@@ -372,7 +372,7 @@ class ModelCheckpoint(MonitorCallback):
         """
         dir_name = os.path.dirname(pattern)
         base_name = os.path.basename(pattern)
-        base_name_regex = "^" + re.sub(r"{.*}", r".*", base_name) + "$"
+        base_name_regex = f"^{re.sub(r'{.*}', r'.*', base_name)}$"
 
         latest_mod_time = 0
         file_path_with_latest_mod_time = None
