@@ -23,7 +23,8 @@ class ELU(Layer):
         super().__init__(**kwargs)
         self.alpha = alpha
         self.supports_masking = True
-        self.built = True
+
+        self._build_at_init()
 
     def call(self, inputs):
         return activations.elu(inputs, alpha=self.alpha)
