@@ -8,6 +8,8 @@ from keras.src import utils
 from keras.src.api_export import keras_export
 from keras.src.layers.layer import Layer
 from keras.src.models.variable_mapping import map_saveable_variables
+from keras.src.quantizers.gptq_config import GPTQConfig
+from keras.src.quantizers.gptq_core import gptq_quantize
 from keras.src.saving import saving_api
 from keras.src.trainers import trainer as base_trainer
 from keras.src.utils import summary_utils
@@ -420,7 +422,7 @@ class Model(Trainer, base_trainer.Trainer, Layer):
             **kwargs,
         )
 
-    def quantize(self, mode, **kwargs):
+    def quantize(self, mode, config=None, **kwargs):
         """Quantize the weights of the model.
 
         Note that the model must be built first before calling this method.
@@ -433,32 +435,61 @@ class Model(Trainer, base_trainer.Trainer, Layer):
         """
         from keras.src.dtype_policies import QUANTIZATION_MODES
 
+        # Validate inputs.
         type_check = kwargs.pop("type_check", True)
         if kwargs:
             raise ValueError(
                 "Unrecognized keyword arguments "
                 f"passed to {self.__class__.__name__}: {kwargs}"
             )
+
         if mode not in QUANTIZATION_MODES:
             raise ValueError(
                 "Invalid quantization mode. "
                 f"Expected one of {QUANTIZATION_MODES}. Received: mode={mode}"
             )
-        mode_changed = False
+
+        if mode == "gptq":
+            if not isinstance(config, GPTQConfig):
+                raise ValueError(
+                    "Mode 'gptq' requires a valid `config` argument of type "
+                    f"`GPTQConfig`. Received: {type(config)}"
+                )
+        elif config is not None:
+            # All other modes must not receive a config
+            raise ValueError(
+                f"The `config` argument is only supported for 'gptq' mode, "
+                f"but received mode='{mode}' and a non-None config."
+            )
+
+        graph_modified = False
         for layer in self._flatten_layers():
-            list_of_sublayers = list(layer._flatten_layers())
-            if len(list_of_sublayers) == 1:  # leaves of the model
+            if len(list(layer._flatten_layers())) == 1:
                 try:
-                    layer.quantize(mode, type_check=type_check)
-                    mode_changed = True
+                    layer.quantize(mode, type_check=type_check, config=config)
+                    graph_modified = True
                 except NotImplementedError as e:
                     warnings.warn(str(e))
-        # We need to set these functions to `None` to remake them for changed
-        # call function
-        if mode_changed:
+                except AttributeError:
+                    pass
+
+        if mode == "gptq":
+            gptq_quantize(self, config)
+
+        # If any layer was changed, we must rebuild the execution functions.
+        if graph_modified:
             self.train_function = None
             self.test_function = None
             self.predict_function = None
+            self._post_quantize(mode, **kwargs)
+
+    def _post_quantize(self, mode, **kwargs):
+        if backend.backend() == "torch":
+            # We need to manually retrack `torch_params`.
+            # The reason is that after quantization, the removed variables are
+            # still referenced by `torch_params` and cannot be gc.
+            for layer in self._flatten_layers():
+                layer._track_variables()
 
     def build_from_config(self, config):
         if not config:
@@ -548,17 +579,20 @@ class Model(Trainer, base_trainer.Trainer, Layer):
                 `tf.TensorSpec`, `backend.KerasTensor`, or backend tensor. If
                 not provided, it will be automatically computed. Defaults to
                 `None`.
-            **kwargs: Additional keyword arguments:
-                - Specific to the JAX backend and `format="tf_saved_model"`:
-                    - `is_static`: Optional `bool`. Indicates whether `fn` is
-                        static. Set to `False` if `fn` involves state updates
-                        (e.g., RNG seeds and counters).
-                    - `jax2tf_kwargs`: Optional `dict`. Arguments for
-                        `jax2tf.convert`. See the documentation for
-                        [`jax2tf.convert`](
-                            https://github.com/google/jax/blob/main/jax/experimental/jax2tf/README.md).
-                        If `native_serialization` and `polymorphic_shapes` are
-                        not provided, they will be automatically computed.
+            **kwargs: Additional keyword arguments.
+                - `is_static`: Optional `bool`. Specific to the JAX backend and
+                    `format="tf_saved_model"`. Indicates whether `fn` is static.
+                    Set to `False` if `fn` involves state updates (e.g., RNG
+                    seeds and counters).
+                - `jax2tf_kwargs`: Optional `dict`. Specific to the JAX backend
+                    and `format="tf_saved_model"`. Arguments for
+                    `jax2tf.convert`. See the documentation for
+                    [`jax2tf.convert`](
+                        https://github.com/google/jax/blob/main/jax/experimental/jax2tf/README.md).
+                    If `native_serialization` and `polymorphic_shapes` are not
+                    provided, they will be automatically computed.
+                - `opset_version`: Optional `int`. Specific to `format="onnx"`.
+                    An integer value that specifies the ONNX opset version.
 
         **Note:** This feature is currently supported only with TensorFlow, JAX
         and Torch backends.
@@ -851,9 +885,9 @@ class Model(Trainer, base_trainer.Trainer, Layer):
         def _flatten(current_dict, prefix=""):
             for key, value in current_dict.items():
                 if isinstance(value, dict):
-                    _flatten(value, prefix + key + "/")
+                    _flatten(value, f"{prefix}{key}/")
                 else:
-                    flat_dict[prefix + key] = value
+                    flat_dict[f"{prefix}{key}"] = value
 
         _flatten(nested_dict)
         return flat_dict
