@@ -214,11 +214,12 @@ class Sequential(Model):
                 raise e
         outputs = x
         self._functional = Functional(inputs=inputs, outputs=outputs)
-        self.built = True
 
-    def call(self, inputs, training=None, mask=None):
+    def call(self, inputs, training=None, mask=None, **kwargs):
         if self._functional:
-            return self._functional.call(inputs, training=training, mask=mask)
+            return self._functional.call(
+                inputs, training=training, mask=mask, **kwargs
+            )
 
         # Fallback: Just apply the layer sequence.
         # This typically happens if `inputs` is a nested struct.
@@ -227,12 +228,17 @@ class Sequential(Model):
             # `outputs` are the outputs of `layer` applied to `inputs`. At the
             # end of each iteration `inputs` is set to `outputs` to prepare for
             # the next layer.
-            kwargs = {}
+            layer_kwargs = {
+                k: kwargs[k]
+                # only inject if this layer’s signature actually has that arg
+                for k in getattr(layer, "_call_has_context_arg", {})
+                if k in kwargs
+            }
             if layer._call_has_mask_arg:
-                kwargs["mask"] = mask
+                layer_kwargs["mask"] = mask
             if layer._call_has_training_arg and training is not None:
-                kwargs["training"] = training
-            outputs = layer(inputs, **kwargs)
+                layer_kwargs["training"] = training
+            outputs = layer(inputs, **layer_kwargs)
             inputs = outputs
 
             mask = tree.map_structure(backend.get_keras_mask, outputs)
@@ -255,15 +261,17 @@ class Sequential(Model):
             "Use `add()` and `pop()` to change the layers in this model."
         )
 
-    def compute_output_spec(self, inputs, training=None, mask=None):
+    def compute_output_spec(self, inputs, training=None, mask=None, **kwargs):
         if self._functional:
             return self._functional.compute_output_spec(
-                inputs, training=training, mask=mask
+                inputs, training=training, mask=mask, **kwargs
             )
         # Direct application
         for layer in self.layers:
             outputs = layer.compute_output_spec(
-                inputs, training=training
+                inputs,
+                training=training,
+                **kwargs,
             )  # Ignore mask
             inputs = outputs
         return outputs
