@@ -9,6 +9,7 @@ from keras.src.backend import any_symbolic_tensors
 from keras.src.backend.common.backend_utils import canonicalize_axis
 from keras.src.backend.common.backend_utils import standardize_axis_for_numpy
 from keras.src.ops.operation import Operation
+from keras.src.quantizers.gptq_config import GPTQConfig
 
 """Int8-related classes and methods"""
 
@@ -377,7 +378,7 @@ def quantize_and_dequantize(inputs, scale, quantized_dtype, compute_dtype):
 
 
 @keras_export("keras.quantizers.pack_int4")
-def pack_int4(arr, axis=0):
+def pack_int4(arr, axis=0, dtype="int8"):
     """Pack an int4 tensor into an int8 tensor with packed nibbles.
 
     The input values must already be int8 in the signed range `[-8, 7]` and
@@ -389,8 +390,11 @@ def pack_int4(arr, axis=0):
     the value from the second row.
 
     Args:
-        arr: An int8 tensor containing int4 values in the range `[-8, 7]`.
+        arr: An `int8` or `uint8` tensor containing int4 values in the range
+            `[-8, 7]`.
         axis: The axis along which to pack the tensor. Defaults to 0.
+        dtype: The data type of the input and packed tensor. Can be
+            `"int8"` or `"uint8"`. Defaults to `"int8"`.
 
     Returns:
         tuple: A tuple `(packed, packed_shape, orig_rows)` where `packed` is
@@ -450,9 +454,14 @@ def pack_int4(arr, axis=0):
     True
     ```
     """
-    if backend.standardize_dtype(arr.dtype) != "int8":
+    if dtype not in ("int8", "uint8"):
+        raise ValueError(
+            f"Expected dtype to be 'int8' or 'uint8', but got '{dtype}'."
+        )
+    if backend.standardize_dtype(arr.dtype) != dtype:
         raise TypeError(
-            "Expected int8 tensor for packing, got {}".format(arr.dtype)
+            f"Expected {dtype} tensor for packing, got "
+            f"{backend.standardize_dtype(arr.dtype)}."
         )
 
     rank = getattr(arr.shape, "rank", None) or len(arr.shape)
@@ -486,12 +495,12 @@ def pack_int4(arr, axis=0):
     low = padded[::2, ...]
     high = padded[1::2, ...]
 
-    mask = ops.array(0x0F, dtype="int8")
+    mask = ops.array(0x0F, dtype=dtype)
     low_u = ops.bitwise_and(low, mask)
     high_u = ops.bitwise_and(high, mask)
 
     packed = ops.bitwise_or(low_u, ops.left_shift(high_u, 4))
-    packed = ops.cast(packed, "int8")
+    packed = ops.cast(packed, dtype)
 
     # 5-6. Restore shape.
     packed = ops.transpose(packed, inv_perm)  # back to original order
@@ -500,7 +509,7 @@ def pack_int4(arr, axis=0):
 
 
 @keras_export("keras.quantizers.unpack_int4")
-def unpack_int4(packed, orig_len, axis=0):
+def unpack_int4(packed, orig_len, axis=0, dtype="int8"):
     """Unpack a packed int4 back to an int8 tensor in the range [-8, 7].
 
     This function reverses the packing performed by `pack_int4`, restoring
@@ -518,6 +527,8 @@ def unpack_int4(packed, orig_len, axis=0):
             packed. This is used to remove any padding that may have
             been added during packing to ensure an even number of rows.
         axis: The axis along which the tensor was packed. Defaults to 0.
+        dtype: The data type of the input and unpacked tensor. Can be
+            `"int8"` or `"uint8"`. Defaults to `"int8"`.
 
     Returns:
         unpacked: An int8 tensor with the same shape as the original
@@ -574,13 +585,24 @@ def unpack_int4(packed, orig_len, axis=0):
     True
     ```
     """
-    if backend.standardize_dtype(packed.dtype) != "int8":
-        raise TypeError(
-            f"Expected int8 tensor for unpacking, got {packed.dtype}"
+    if dtype not in ("int8", "uint8"):
+        raise ValueError(
+            f"Expected dtype to be 'int8' or 'uint8', but got '{dtype}'."
         )
 
-    rank = getattr(packed.shape, "rank", None) or len(packed.shape)
+    if backend.standardize_dtype(packed.dtype) not in ("int8", "uint8"):
+        raise TypeError(
+            f"Expected int8 or uint8 tensor for unpacking, got {packed.dtype}"
+        )
 
+    def to_signed(x):
+        """Converts unpacked nibbles [0, 15] to signed int4 [-8, 7]."""
+        dtype_x = backend.standardize_dtype(x.dtype)
+        eight = ops.cast(8, dtype_x)
+        sixteen = ops.cast(16, dtype_x)
+        return ops.where(x < eight, x, x - sixteen)
+
+    rank = getattr(packed.shape, "rank", None) or len(packed.shape)
     if axis < 0:
         axis += rank
 
@@ -591,16 +613,15 @@ def unpack_int4(packed, orig_len, axis=0):
         low_unpacked = ops.bitwise_and(packed, mask)
         high_unpacked = ops.bitwise_and(ops.right_shift(packed, 4), mask)
 
-        # Convert values from [0, 15] to [-8, 7].
-        low_signed = ops.where(
-            low_unpacked < 8, low_unpacked, low_unpacked - 16
-        )
-        high_signed = ops.where(
-            high_unpacked < 8, high_unpacked, high_unpacked - 16
-        )
+        if dtype == "int8":
+            low_unpacked = to_signed(low_unpacked)
+            high_unpacked = to_signed(high_unpacked)
+
+        low_final = ops.cast(low_unpacked, dtype)
+        high_final = ops.cast(high_unpacked, dtype)
 
         # Interleave and reshape
-        stacked = ops.stack([low_signed, high_signed], axis=1)
+        stacked = ops.stack([low_final, high_final], axis=1)
         unpacked = ops.reshape(stacked, (-1,) + tuple(ops.shape(packed)[1:]))
 
         # Remove padding and return
@@ -612,25 +633,307 @@ def unpack_int4(packed, orig_len, axis=0):
     transposed = ops.transpose(packed, perm)
 
     # 1. Split nibbles.
-    mask = ops.array(0x0F, dtype="int8")  # int8 arrays
+    mask = ops.array(0x0F, dtype=packed.dtype)
     low = ops.bitwise_and(transposed, mask)
     high = ops.bitwise_and(ops.right_shift(transposed, 4), mask)
 
-    eight = ops.array(8, dtype="int8")
-    sixteen = ops.array(16, dtype="int8")
+    # 2. Conditionally convert to signed.
+    if dtype == "int8":
+        low = to_signed(low)
+        high = to_signed(high)
 
-    def to_signed(x):
-        return ops.where(x < eight, x, x - sixteen)
+    low = ops.cast(low, dtype)
+    high = ops.cast(high, dtype)
 
-    low = to_signed(low)
-    high = to_signed(high)
-
-    # 2. Interleave and reshape.
-    stacked = ops.stack([low, high], axis=1)  # (pairs, 2, ...)
+    # 3. Interleave and reshape.
+    stacked = ops.stack([low, high], axis=1)
     unpacked = ops.reshape(stacked, (-1,) + tuple(ops.shape(transposed)[1:]))
 
     # 4. Remove padding and restore original layout.
     unpacked = unpacked[:orig_len, ...]
     unpacked = ops.transpose(unpacked, inv_perm)
 
-    return unpacked  # dtype is int8
+    return unpacked
+
+
+class GPTQQuantizer(Quantizer):
+    """A class that handles the quantization of weights using GPTQ method.
+
+    This class provides methods to find quantization parameters (scale and zero)
+    for a given tensor and can be used to quantize weights in a GPTQ context.
+
+    Args:
+        weight_bits: (int) The number of bits to quantize to (e.g., 4).
+        per_channel: (bool) A flag indicating whether quantization is
+            applied per-channel (`True`) or per-tensor (`False`).
+            Defaults to `False`.
+        symmetric: (bool) A flag indicating whether symmetric (`True`) or
+            asymmetric (`False`) quantization is used. Defaults to `False`.
+        group_size: (int) The size of weight groups for quantization. A
+            value of -1 indicates that grouping is not used.
+            Defaults to -1.
+    """
+
+    def __init__(
+        self,
+        config=GPTQConfig(tokenizer=None, dataset=None),
+        compute_dtype="float32",
+    ):
+        Quantizer.__init__(self)
+        self.weight_bits = config.weight_bits
+        self.per_channel = config.per_channel
+        self.symmetric = config.symmetric
+        self.group_size = config.group_size
+        self.compute_dtype = compute_dtype
+
+        # These are now determined later by `find_params`
+        self.scale = None
+        self.zero = None
+        self.maxq = None
+
+    def find_params(self, input_tensor, weight=True):
+        """Finds quantization parameters (scale and zero) for a given tensor."""
+        self.scale, self.zero, self.maxq = compute_quantization_parameters(
+            input_tensor,
+            bits=self.weight_bits,
+            symmetric=self.symmetric,
+            per_channel=self.per_channel,
+            group_size=self.group_size,
+            weight=weight,
+            compute_dtype=self.compute_dtype,
+        )
+        return self.scale, self.zero, self.maxq
+
+    def get_config(self):
+        config = super().get_config()
+        config.update(
+            {
+                "weight_bits": self.weight_bits,
+                "per_channel": self.per_channel,
+                "symmetric": self.symmetric,
+                "group_size": self.group_size,
+            }
+        )
+        return config
+
+    @classmethod
+    def from_config(cls, config):
+        gptq = GPTQConfig(
+            tokenizer=None,
+            dataset=None,
+            weight_bits=config["weight_bits"],
+            per_channel=config["per_channel"],
+            symmetric=config["symmetric"],
+            group_size=config["group_size"],
+        )
+        return cls(gptq)
+
+
+def compute_quantization_parameters(
+    x,
+    *,
+    bits,
+    symmetric=False,
+    per_channel=False,
+    group_size=-1,
+    weight=False,
+    compute_dtype="float32",
+):
+    """
+    Computes the scale and zero-point for quantization.
+
+    This function calculates the scale and zero-point required for quantizing
+    a given tensor `x` based on the specified parameters. It supports grouped,
+    per-channel, per-tensor, symmetric, and asymmetric quantization - along
+    with any combinations of these.
+
+    Args:
+        x: KerasTensor. The input tensor to quantize.
+        bits: int. The number of bits to quantize to (e.g., 4).
+        symmetric: bool. Whether to use symmetric quantization.
+        per_channel: bool. Whether to quantize per channel.
+        group_size: int. The group size for quantization.
+        weight: bool. Whether the input tensor is a weight tensor.
+
+    Returns:
+        scale: KerasTensor. The scale tensor for quantization.
+        zero: KerasTensor. The zero tensor for quantization.
+        maxq: scalar. The maximum quantization value.
+    """
+    if x is None:
+        raise ValueError(f"Input tensor {x} cannot be None.")
+
+    # For weights, we typically expect at least a 2D tensor.
+    if weight and len(x.shape) < 2:
+        raise ValueError(
+            f"Input weight tensor {x} must have a rank of at "
+            f"least 2, but got rank {len(x.shape)}."
+        )
+
+    if ops.size(x) == 0:
+        raise ValueError("Input tensor 'x' cannot be empty.")
+
+    original_shape = x.shape
+
+    if per_channel:
+        if weight:
+            if group_size != -1:
+                input_reshaped = ops.reshape(x, [-1, group_size])
+            else:
+                input_reshaped = ops.reshape(x, [original_shape[0], -1])
+    else:  # per-tensor
+        input_reshaped = ops.reshape(x, [1, -1])
+
+    # Find min/max values
+    min_values = ops.min(input_reshaped, axis=1)
+    max_values = ops.max(input_reshaped, axis=1)
+
+    # Apply symmetric quantization logic if enabled
+    if symmetric:
+        max_values = ops.maximum(ops.abs(min_values), max_values)
+        min_values = ops.where(
+            ops.less(min_values, 0), ops.negative(max_values), min_values
+        )
+
+    # Ensure range is not zero to avoid division errors
+    zero_range = ops.equal(min_values, max_values)
+    min_values = ops.where(zero_range, ops.subtract(min_values, 1), min_values)
+    max_values = ops.where(zero_range, ops.add(max_values, 1), max_values)
+
+    maxq = ops.cast(ops.subtract(ops.power(2, bits), 1), compute_dtype)
+
+    # Calculate scale and zero-point
+    scale = ops.divide(ops.subtract(max_values, min_values), maxq)
+    if symmetric:
+        zero = ops.full_like(scale, ops.divide(ops.add(maxq, 1), 2))
+    else:
+        zero = ops.round(ops.divide(ops.negative(min_values), scale))
+
+    # Ensure scale is non-zero
+    scale = ops.where(ops.less_equal(scale, 0), 1e-8, scale)
+
+    if weight:
+        # Per-channel, non-grouped case: simple reshape is correct.
+        if per_channel and group_size == -1:
+            scale = ops.reshape(scale, [-1, 1])
+            zero = ops.reshape(zero, [-1, 1])
+        elif not per_channel:
+            num_rows = original_shape[0]
+            scale = ops.tile(ops.reshape(scale, (1, 1)), (num_rows, 1))
+            zero = ops.tile(ops.reshape(zero, (1, 1)), (num_rows, 1))
+    if per_channel:
+        scale = ops.reshape(scale, [-1, 1])
+        zero = ops.reshape(zero, [-1, 1])
+
+    zero = ops.cast(zero, "uint8")
+
+    return scale, zero, maxq
+
+
+def quantize_with_zero_point(input_tensor, scale, zero, maxq):
+    """Quantize a float tensor into discrete levels [0, maxq] using
+    per-tensor/per-channel/grouped scaling.
+
+    Returns `q` (same dtype as inputs/scales; float is fine) where values are in
+    [0, maxq].
+
+    Args:
+        input_tensor: KerasTensor. The input tensor to quantize.
+        scale: KerasTensor. The scale tensor for quantization.
+        zero: KerasTensor. The zero tensor for quantization.
+        maxq: KerasTensor. The maximum quantization value.
+
+    Returns:
+        KerasTensor. The quantized tensor.
+    """
+    # Guard against divide-by-zero
+    epsilon = ops.cast(1e-8, dtype=scale.dtype)
+    safe_scale = ops.where(ops.equal(scale, 0), epsilon, scale)
+
+    quantized_tensor = ops.round(
+        ops.add(
+            ops.divide(input_tensor, safe_scale), ops.cast(zero, scale.dtype)
+        )
+    )
+    quantized_tensor = ops.clip(quantized_tensor, 0, maxq)
+    return quantized_tensor
+
+
+def dequantize_with_zero_point(input_tensor, scale, zero):
+    """
+    Dequantizes a quantized tensor using the provided scale and zero tensors.
+
+    Args:
+        input_tensor: KerasTensor. The quantized tensor to dequantize.
+        scale: KerasTensor. The scale tensor for dequantization.
+        zero: KerasTensor. The zero tensor for dequantization.
+
+    Returns:
+        KerasTensor. The dequantized tensor.
+    """
+    return ops.multiply(
+        scale, ops.subtract(input_tensor, ops.cast(zero, scale.dtype))
+    )
+
+
+def quantize_with_sz_map(weights_matrix, scale, zero, g_idx, maxq):
+    """Quantize the weight matrix from group params.
+
+    This function uses the provided scale and zero tensors to quantize the
+    input weights_matrix according to the group indices. It maps each column
+    of the weights_matrix to its corresponding group parameters and performs
+    the quantization operation.
+
+    Args:
+        weights_matrix: 2D tensor of shape [out_features, in_features].
+        scale: Per-group scale tensor of shape [out_features, n_groups].
+        zero: Per-group zero-point tensor of shape [out_features, n_groups].
+        g_idx: Integer tensor of shape [in_features,] mapping each column to
+            its group index.
+        maxq: Scalar (float) representing the maximum integer quantization
+            level (e.g., 2^bits - 1).
+
+    Returns:
+        A tensor with the same shape as `weights_matrix` containing the
+        quantized weights produced using the provided group parameters.
+    """
+    groups = ops.cast(g_idx, "int32")
+    scale_cols = ops.take(scale, groups, axis=1)  # [out_features, in_features]
+    zero_cols = ops.take(zero, groups, axis=1)  # [out_features, in_features]
+
+    # Quantize elementwise, then cast to int
+    return quantize_with_zero_point(weights_matrix, scale_cols, zero_cols, maxq)
+
+
+def dequantize_with_sz_map(weights_matrix, scale, zero, g_idx):
+    """Rebuild a dequantized weight matrix from group params.
+
+    This function uses the provided scale and zero tensors to dequantize the
+    input weights_matrix according to the group indices. It maps each column
+    of the weights_matrix to its corresponding group parameters and performs
+    the dequantization operation.
+
+    Args:
+        weights_matrix: 2D tensor of shape [out_features, in_features].
+        scale: Per-group scale tensor of shape [out_features, n_groups].
+        zero: Per-group zero-point tensor of shape [out_features, n_groups].
+        g_idx: Integer tensor of shape [in_features,] mapping each column to
+            its group index.
+        maxq: Scalar (float) representing the maximum integer quantization
+            level (e.g., 2^bits - 1).
+
+    Returns:
+        A tensor with the same shape as `weights_matrix` containing the
+        dequantized weights produced using the provided group parameters.
+    """
+    # Map group indices to scales and zeros
+    groups = ops.cast(g_idx, "int32")
+    scales_mapped = ops.take(scale, groups, axis=1)
+    zeros_mapped = ops.take(zero, groups, axis=1)
+    zeros_mapped = ops.cast(zeros_mapped, scales_mapped.dtype)
+
+    quantized = ops.multiply(
+        ops.subtract(weights_matrix, zeros_mapped), scales_mapped
+    )
+
+    return quantized
