@@ -181,6 +181,8 @@ def pythonify_logs(logs):
         A flattened dict with values converted to Python-native types if
         possible.
     """
+    from keras.src import backend
+
     logs = logs or {}
     result = {}
     for key, value in sorted(logs.items()):
@@ -188,6 +190,9 @@ def pythonify_logs(logs):
             result.update(pythonify_logs(value))
         else:
             try:
+                # Prevent torch compiler from breaking the graph.
+                if backend.is_tensor(value):
+                    value = backend.convert_to_numpy(value)
                 value = float(value)
             except:
                 pass
